@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1990, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 2000, 2009, 2010, 2011, 2012, 2013, 2016, 2019
+ * Copyright (c) 1990, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 2000, 2009, 2010, 2011, 2012, 2013, 2016, 2019, 2022, 2023
  * The Regents of the University of California. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -26,11 +26,11 @@
  * SUCH DAMAGE.
  */
 #ifndef lint
-static const char copyright[] =
-    "Copyright (c) 1990, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 2000, 2009, 2010, 2011, 2012, 2013, 2016, 2019\n\
+static const char copyright[] __attribute__((unused)) =
+    "Copyright (c) 1990, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 2000, 2009, 2010, 2011, 2012, 2013, 2016, 2019, 2022, 2023\n\
 The Regents of the University of California.  All rights reserved.\n";
-static const char rcsid[] =
-    "@(#) $Id: arpwatch.c 1505 2019-11-30 18:35:23Z leres $ (LBL)";
+static const char rcsid[] __attribute__((unused)) =
+    "@(#) $Id: arpwatch.c 1540 2023-12-03 18:09:52Z leres $ (LBL)";
 #endif
 
 /*
@@ -123,11 +123,12 @@ const char *watcher = WATCHER;
 const char *watchee = WATCHEE;
 
 /* Locals */
-int can_checkpoint;
-int swapped;
+int can_checkpoint = 1;
+int docheckpoint;
+int foreground;
 int nobogons;
 int promisc = 1;
-int foreground;
+int swapped;
 
 static u_int32_t net;
 static u_int32_t netmask;
@@ -157,8 +158,9 @@ extern char *optarg;
 
 /* Forwards */
 int	addnet(enum action action, const char *);
-RETSIGTYPE checkpoint(int);
-RETSIGTYPE die(int);
+void	checkpoint(int);
+void	die(int);
+void	hup(int);
 int	isbogon(u_int32_t);
 int	main(int, char **);
 void	process_ether(u_char *, const struct pcap_pkthdr *, const u_char *);
@@ -421,7 +423,7 @@ main(int argc, char **argv)
 
 	(void)setsignal(SIGINT, die);
 	(void)setsignal(SIGTERM, die);
-	(void)setsignal(SIGHUP, die);
+	(void)setsignal(SIGHUP, hup);
 	if (rfilename == NULL) {
 		(void)setsignal(SIGQUIT, checkpoint);
 		(void)setsignal(SIGALRM, checkpoint);
@@ -514,6 +516,11 @@ process_ether(u_char *u, const struct pcap_pkthdr *h, const u_char *p)
 		lg(LOG_ERR, "ent_add(%s, %s, %ld) failed",
 		    intoa(sia), e2str(sea), (u_long)t);
 	can_checkpoint = 1;
+
+	if (docheckpoint) {
+		checkpoint(0);
+		docheckpoint = 0;
+	}
 }
 
 /* Perform sanity checks on an ethernet arp/rarp packet, return true if ok */
@@ -669,6 +676,11 @@ process_fddi(u_char *u, const struct pcap_pkthdr *h, const u_char *p)
 		lg(LOG_ERR, "ent_add(%s, %s, %ld) failed",
 		    intoa(sia), e2str(sea), (u_long)t);
 	can_checkpoint = 1;
+
+	if (docheckpoint) {
+		checkpoint(0);
+		docheckpoint = 0;
+	}
 }
 
 /* Perform sanity checks on arp/rarp packet, return true if ok */
@@ -684,9 +696,9 @@ sanity_fddi(struct fddi_header *fh, struct ether_arp *ea, int len)
 	 * as shorts */
 
 	memmove(&type, fh->snap.snap_type, sizeof(u_short));
-	memmove(&(ea->arp_hrd), &hrd, sizeof(hrd));
-	memmove(&(ea->arp_pro), &pro, sizeof(pro));
-	memmove(&(ea->arp_op), &op, sizeof(op));
+	memmove(&hrd, &(ea->arp_hrd), sizeof(hrd));
+	memmove(&pro, &(ea->arp_pro), sizeof(pro));
+	memmove(&op, &(ea->arp_op), sizeof(op));
 
 	type = ntohs(type);
 	hrd = ntohs(hrd);
@@ -844,7 +856,7 @@ addnet(enum action action, const char *str)
 		nets_size = size;
 		nets_ind = ind;
 		nets = ns;
-	} else if ( action == SKIP) {
+	} else if (action == SKIP) {
 		skipnets_size = size;
 		skipnets_ind = ind;
 		skipnets = ns;
@@ -886,19 +898,9 @@ isbogon(u_int32_t sia)
 	return (1);
 }
 
-RETSIGTYPE
-die(int signo)
-{
-
-	lg(LOG_DEBUG, "exiting");
-	checkpoint(0);
-	exit(1);
-}
-
-RETSIGTYPE
+void
 checkpoint(int signo)
 {
-
 	if (!can_checkpoint)
 		(void)alarm(1);
 	else {
@@ -906,7 +908,23 @@ checkpoint(int signo)
 		(void)dump();
 		(void)alarm(CHECKPOINT);
 	}
-	return RETSIGVAL;
+}
+
+void
+die(int signo)
+{
+	lg(LOG_DEBUG, "exiting");
+	checkpoint(0);
+	exit(1);
+}
+
+void
+hup(int signo)
+{
+	if (can_checkpoint)
+		checkpoint(0);
+	else
+		docheckpoint = 1;
 }
 
 void
@@ -921,7 +939,6 @@ lg(int pri, const char *fmt, ...)
 void
 usage(void)
 {
-
 	(void)fprintf(stderr, "Version %s\n", version);
 	(void)fprintf(stderr,
 	    "usage: %s [-CdFNpqsvzZ] [-D arpdir] [-f datafile]"
